import sys
import time
import File_creation.file_creation as fc
import Parser.read_input as ri
import Main_functions.main_functions as mf
import Partition.create_elements as ce
import Abstraction.weighted_graph as abs
import Model_check.verification as mc
import networkx as nx
import matplotlib.pyplot as plt




""" INPUT for the Main program """
""" Kind of hybrid automaton (linear or polyhedral). """
#HA_type = raw_input('What kind of hybrid automaton? (linear/polyhedral)\n')
# This is just a version for polyhedral switched systems
HA_type = 'polyhedral'

""" Name folder where the input folder containing the input automaton text file can be found. """
namefolder = raw_input('\n* Please specify the path for the folder in which the experiment data (input.dat) is stored:\n')
#namefolder = '/Users/mgarcia/Averist_code_for_webpage/Experiments/polyhedral_dyn/3D_unstable'


am_answer = raw_input('\n* Do you want the linear expressions for creating the regions to be generated automatically (A) or do you want to add them manually (M)? Enter A/M:\n')

if am_answer == 'A':
	
	given_linear_exp = 'nle'
	unif_linear_exp = int(raw_input('\n* The linear expressions will be generated in a uniform fashion. Please specify the granularity -- a natural number (higher number indicates finer partition):\n'))


elif am_answer == 'M':

	given_linear_exp = 'le'
	unif_linear_exp
	print ('\nPlease specify the linear expressions for generating the regions in linearexp.dat\n')

else:
	sys.exit('Wrong answer')

extract_answer = raw_input('\n* In addition, do you want to add the linear expressions appearing in the input hybrid automaton? Enter Y/N:\n')

if extract_answer == 'Y':
	extract_linear_exp = 'exle'
elif extract_answer == 'N':
	extract_linear_exp = 'nexle'
else:
	sys.exit('Wrong answer')



total_time = time.time()

""" Creates the necessary folder to store general information about the results. """
fc.folders(namefolder)

""" Reads the input automaton. """
var_dict,inv_var_dict,P,G = ri.read_input(namefolder+'/input.dat',HA_type)

""" Stores the input automaton graph into a file. """
fc.store_graph(G,namefolder,HA_type)


""" Construction of the linear expression matrix. """
LE = mf.initial_LE(P,given_linear_exp,extract_linear_exp,unif_linear_exp,inv_var_dict,namefolder)

""" Creates the necessary folders to store information about the results. """
namefolder_le = fc.itfolders(LE,namefolder)

""" Stores the linear expressions into a file. """
fc.store_le(LE,namefolder_le+'/output')

""" Element creation for partition. """
E, element_creation_time = ce.create_elements(LE)

""" Stores the elements into a file. """
fc.store_elements(E,namefolder_le+'/output')

""" Checking if there is explosion in some element. In this case a further analysis
is not needed, instability of the input hybrid system is obtained. """
explosion = mf.check_explosion(G)


""" In case of blow-up the program has finished. """
if explosion:
	""" Set the times. """
	abstraction_time = 0
	subgraph_time = 0
	optimization_time = 0
	verification_time = 0

	""" Stability answer (blow-up). """
	stability_msg = 'Unstable (blow-up)'
	#fc.store_stable_answer(stability_msg,[],namefolder_le+'/output')


else:

	""" Constructs the abstract graph. """
	abstraction_time = time.time()
	WG,subgraph_time,optimization_time = abs.construct_weighted_graph(G,E)
	abstraction_time = time.time() - abstraction_time

	""" Stores the abstract graph. """
	fc.store_WG(WG,namefolder_le+'/output/weighted_graph.dat')
	

	""" Store the weighted graph draw. """
	nx.draw(WG)
	plt.savefig(namefolder_le+'/output/weighted_graph.png')
	
	""" Verification of the weighted abstract graph. It returns a verification message of stability
		in the input hybrid automaton or abstract counterexample found. In case there is an abstract
		counterexample, this is also returned. The abstract counterexample is a cycle in the weighted
		abstract graph with total weight greater than one. """
	verification_time = time.time()
	stability_msg,counterexample = mc.verification(WG)
	verification_time = time.time() - verification_time


""" Store the stability answer. """
fc.store_stable_answer(stability_msg,counterexample,namefolder_le+'/output')

""" Stores the times. """
total_time = time.time() - total_time
fc.store_time(element_creation_time,abstraction_time,subgraph_time,optimization_time,verification_time,total_time,namefolder_le)


print '\nSTABILITY ANSWER =', stability_msg








	
		

	



