import File_creation.functions as fun
import Partition.create_linear_exp as cle
import PPL_functions.ppl_functions as pplf
from sage.libs.ppl import *
import sys



#----------------------------------------------------------------------------------------------#
def initial_LE(P,given_linear_exp,extract_linear_exp,unif_linear_exp,inv_var_dict,namefolder):
	""" Construction of the linear expression matrix for the first CEGAR iteration. """
	
	strLE = []

	if given_linear_exp == 'le':
		strLE = fun.load(namefolder+'/linearexp.dat')

	if extract_linear_exp == 'exle':
		PLE = fun.transform_P(P)
		strLE = strLE + PLE

		
	if unif_linear_exp > -1:
		ULE = cle.linear_expressions(unif_linear_exp,inv_var_dict)
		strLE = strLE + ULE


	if not strLE:
		sys.exit('There is no any linear expression to partition the state-space')

	strLE = list(set(strLE))

	# Now LE is transformed into a list of ppl.linear expressions
	LE = pplf.get_linearexps(strLE,inv_var_dict)

	return LE



#----------------------------------------------------------------------------------------------#
def check_explosion(G):
	
	""" Checks for the case of explosion, by asking if the invariants intersect with its dynamics 
	polyhedron. """
	
	explosion = False

	for n in G.nodes():
		invpoly = NNC_Polyhedron(G.node[n]['inv'])
		#print 'invpoly =', invpoly.constraints()
		dynpoly = G.node[n]['dyn']
		# Check if dynpoly is just the point zero. In case of being zero there is no movement
		# and therefore explosion is not possible.
		if not pplf.is_zero(dynpoly):
			# Compute the closed dynamics polyhedron (to avoid a not bounded problem in optimization)
			cdynpoly = pplf.copy_closed_polyhedron(dynpoly)
			invpoly.intersection_assign(cdynpoly)
			if not invpoly.is_empty() and not pplf.is_zero(invpoly):
				explosion = True
				break

	return explosion


